import sys ,os 
import math 
import subprocess 
from PySide6 .QtWidgets import QApplication ,QMainWindow ,QPushButton ,QLabel ,QWidget ,QMessageBox ,QDialog ,QVBoxLayout ,QMenuBar ,QMenu ,QInputDialog 
from PySide6 .QtGui import QPixmap ,QIcon ,QAction ,QPalette 
from PySide6 .QtCore import Qt ,QSize ,QDateTime ,QSettings 
from cryptography .fernet import Fernet 
import requests 
import dateutil .parser 

DEVELOPMENT_MODE =True 

def verify_activation_code (activation_code ):
    url ="https://1fmjoam3kk.execute-api.ap-southeast-2.amazonaws.com/default/check-authentication"
    data ={"activation_code":activation_code }
    headers ={"Content-Type":"application/json"}

    try :
        response =requests .post (url ,json =data ,headers =headers )
        if response .status_code ==200 :
            response_data =response .json ()
            is_valid =response_data .get ("is_valid",False )
            is_indefinite =response_data .get ("is_indefinite",False )
            expiration_date_str =response_data .get ("expiration_date")

            if expiration_date_str :
                try :
                    expiration_datetime =dateutil .parser .parse (expiration_date_str )
                    expiration_date =QDateTime (
                    expiration_datetime .year ,
                    expiration_datetime .month ,
                    expiration_datetime .day ,
                    expiration_datetime .hour ,
                    expiration_datetime .minute ,
                    expiration_datetime .second ,
                    expiration_datetime .microsecond //1000 ,
                    Qt .LocalTime 
                    )
                except ValueError :
                    print (f"Error: Invalid expiration date format '{expiration_date_str}'")

                    expiration_date =None 
            else :
                expiration_date =QDateTime .currentDateTime ().addYears (100 )
                is_indefinite =False 


            return is_valid ,is_indefinite ,expiration_date 
        else :
            print (f"Error: {response.status_code} - {response.text}")
            return False ,False ,None 
    except requests .exceptions .RequestException as e :
        print (f"Error: {str(e)}")
        return False ,False ,None 

def get_application_path ():
    """Get the directory where the application is running."""
    if getattr (sys ,'frozen',False ):

        application_path =os .path .dirname (sys .executable )
    else :

        application_path =os .path .dirname (os .path .abspath (__file__ ))
    return application_path 

class NanoSenseApp (QMainWindow ):
    def __init__ (self ):
        super ().__init__ ()
        self .setWindowTitle ('NanoSense - A Suite of Nanopore Data Analysis Tools')
        self .setGeometry (100 ,100 ,700 ,850 )
        self .central_widget =QWidget (self )
        self .setCentralWidget (self .central_widget )


        self .setStyleSheet ("background-color: #a7c1cf;")


        nanoSense_html =('<p style="font-size: 35px; color: #154360; margin-top: 0; margin-bottom: 0;">NanoSense</p>'
        '<p style="font-size: 15px; margin: 2px; color: #154360;">A Suite of Nanopore Data Analysis Tools</p>')
        self .title_label =QLabel (nanoSense_html ,self .central_widget )
        self .title_label .setAlignment (Qt .AlignmentFlag .AlignCenter )


        self .image_label =QLabel (self .central_widget )

        application_path =get_application_path ()


        image_path =os .path .join (application_path ,'image_1.jpg')


        pixmap =QPixmap (image_path )
        self .image_label .setPixmap (pixmap .scaled (400 ,400 ,Qt .AspectRatioMode .KeepAspectRatio ))
        self .image_label .setAlignment (Qt .AlignmentFlag .AlignCenter )


        self .button_radius =240 
        self .button_size =QSize (210 ,60 )
        self .button_names =['Data Visualisation','Frequency and multi-plots','Event Analysis','Combine Datasets and Files','Clustering and Data Reduction','ML Analysis','Spectrogram and PSD','Nanopore Size Calc','Resource Monitor','Reduction Settings Viewer','Plotting and selecting','Data Reduction']

        self .buttons =[]


        self .create_circular_buttons ()


        info_html =(
        '<p style="font-size: 18px; margin: 2px; color: #1F618D;">Shankar Dutt</p>'
        '<p style="font-size: 15px; margin: 2px; color: #1F618D;">shankar.dutt@anu.edu.au</p>'
        )
        self .info_label =QLabel (info_html ,self .central_widget )
        self .info_label .setAlignment (Qt .AlignmentFlag .AlignCenter )


        self .resize_widgets ()


        menubar =QMenuBar ()


        options_menu =QMenu ("Options",self )
        for button_name in self .button_names :
            if button_name !="Help":
                action =QAction (button_name ,self )
                action .triggered .connect (lambda _ ,app_name =button_name :self .start_application (app_name ))
                options_menu .addAction (action )
        menubar .addMenu (options_menu )




        help_menu =QMenu ("Help",self )
        help_action =QAction ("Help",self )
        help_action .triggered .connect (self .show_help_dialog )
        help_menu .addAction (help_action )
        menubar .addMenu (help_menu )


        activation_menu =QMenu ("Activation Status",self )
        activation_action =QAction ("Check Activation Status",self )
        activation_action .triggered .connect (self .show_activation_status )
        activation_menu .addAction (activation_action )

        if DEVELOPMENT_MODE :

            reset_activation_action =QAction ("Reset Activation",self )
            reset_activation_action .triggered .connect (self .reset_activation_status )
            activation_menu .addAction (reset_activation_action )

        menubar .addMenu (activation_menu )

        self .setMenuBar (menubar )


        icon_path =os .path .join (application_path ,"icons.icns"if sys .platform =="darwin"else "icons.ico")
        self .setWindowIcon (QIcon (icon_path ))


        self .check_activation_status ()


    def create_circular_buttons (self ):
        button_style ="""
            QPushButton {
                color: white;
                background-color: #3b3b6d;
                border-radius: 30px;
                font: bold 12px;
                padding: 10px;
            }
            QPushButton:hover {
                background-color: #505090;
            }
        """
        for i ,name in enumerate (self .button_names ):
            button =QPushButton (name ,self .central_widget )
            button .setStyleSheet (button_style )
            button .setFixedSize (self .button_size )
            self .buttons .append (button )

            if name !="Help":
                button .clicked .connect (lambda _ ,app_name =name :self .start_application (app_name ))
            else :
                button .clicked .connect (self .show_help_dialog )

    def resize_widgets (self ):

        self .title_label .setGeometry (
        10 ,
        5 ,
        self .width ()-20 ,
        80 
        )


        image_size =self .image_label .pixmap ().size ()
        self .image_label .setGeometry (
        (self .width ()-image_size .width ())//2 ,
        (self .height ()-image_size .height ())//2 ,
        image_size .width (),
        image_size .height ()
        )

        self .button_angles =[0 ,-27 ,-60 ,-90 ,-120 ,-153 ,-36 *5 ,-207 ,-240 ,-270 ,-300 ,-333 ]


        assert len (self .button_angles )==len (self .buttons ),"The number of angles must match the number of buttons."

        self .button_radii =[self .button_radius ]*len (self .buttons )
        self .button_radii [3 ]=self .button_radius +50 
        self .button_radii [9 ]=self .button_radius +40 

        center_x =self .width ()/2 
        center_y =self .height ()/2 -25 

        for i ,button in enumerate (self .buttons ):
            angle_degrees =self .button_angles [i ]
            angle_radians =math .radians (angle_degrees )

            button_radius =self .button_radii [i ]

            x =center_x +math .cos (angle_radians )*button_radius -self .button_size .width ()/2 
            y =center_y -math .sin (angle_radians )*button_radius -self .button_size .height ()/2 

            button .setGeometry (int (x ),int (y ),self .button_size .width (),self .button_size .height ())


        self .info_label .setGeometry (
        5 ,
        self .height ()-100 ,
        self .width ()-10 ,
        70 
        )

    def start_application (self ,app_name ):
        if self .is_activated ():
            app_path =os .path .join (get_application_path (),app_name ,"main.py")
            subprocess .Popen (["python",app_path ])
        else :
            QMessageBox .warning (self ,"Activation Required","Please enter a valid activation code to use this feature.")

    def show_help_dialog (self ):
        dialog =QDialog (self )
        dialog .setWindowTitle ("Help")
        dialog .setStyleSheet ("""
            QDialog {
                background-color: #ffffff;
            }
            QLabel {
                color: #333333;
                font-size: 14px;
            }
        """)
        layout =QVBoxLayout (dialog )

        image_label =QLabel (dialog )
        pixmap =QPixmap (os .path .join (os .path .dirname (os .path .abspath (__file__ )),"image_1.jpg"))
        image_label .setPixmap (pixmap .scaled (200 ,200 ,Qt .AspectRatioMode .KeepAspectRatio ))
        image_label .setAlignment (Qt .AlignmentFlag .AlignCenter )
        layout .addWidget (image_label )

        help_text =QLabel ("For Help, please contact Shankar Dutt (shankar.dutt@anu.edu.au)",dialog )
        help_text .setAlignment (Qt .AlignmentFlag .AlignCenter )
        layout .addWidget (help_text )

        dialog .exec ()


    def resizeEvent (self ,event ):
        self .resize_widgets ()

        super ().resizeEvent (event )

    def check_activation_status (self ):
        settings =QSettings ("NanoSense","ActivationStatus")
        activation_code =settings .value ("activation_code")
        expiration_date =settings .value ("expiration_date")
        is_indefinite =settings .value ("is_indefinite",False )

        if activation_code and expiration_date :
            if is_indefinite or QDateTime .currentDateTime ()<QDateTime .fromString (expiration_date ,Qt .ISODate ):
                return 


        self .prompt_activation_code ()

    def prompt_activation_code (self ):
        while True :
            activation_code ,ok =QInputDialog .getText (self ,"Activation","Please enter your activation code.\n\nIf you do not have an activation code, please contact Shankar Dutt (shankar.dutt@anu.edu.au)")
            if not ok :
                sys .exit ()

            is_valid ,is_indefinite ,expiration_date =verify_activation_code (activation_code )


            if is_valid :

                settings =QSettings ("NanoSense","ActivationStatus")
                settings .setValue ("activation_code",activation_code )
                if is_indefinite :
                    settings .setValue ("expiration_date",None )
                    settings .setValue ("is_indefinite",True )
                else :
                    settings .setValue ("expiration_date",expiration_date .toString (Qt .ISODate ))
                    settings .setValue ("is_indefinite",False )
                break 
            else :
                QMessageBox .warning (self ,"Invalid Code","The entered activation code is invalid. Please try again.")
    def is_activated (self ):
        settings =QSettings ("NanoSense","ActivationStatus")
        activation_code =settings .value ("activation_code")
        expiration_date =settings .value ("expiration_date")

        if activation_code and expiration_date :
            expiration_date =QDateTime .fromString (expiration_date ,Qt .ISODate )
            current_time =QDateTime .currentDateTime ()

            if current_time <expiration_date :
                return True 

        return False 

    def reset_activation_status (self ):
        settings =QSettings ("NanoSense","ActivationStatus")
        settings .remove ("activation_code")
        settings .remove ("activation_time")
        QMessageBox .information (self ,"Reset Activation","Activation status has been reset.")

    def show_activation_status (self ):
        settings =QSettings ("NanoSense","ActivationStatus")
        activation_code =settings .value ("activation_code")
        expiration_date =settings .value ("expiration_date")

        if activation_code and expiration_date :
            expiration_date =QDateTime .fromString (expiration_date ,Qt .ISODate )
            current_time =QDateTime .currentDateTime ()

            if current_time <expiration_date :
                days_remaining =current_time .daysTo (expiration_date )
                if expiration_date .date ()==QDateTime .currentDateTime ().addYears (100 ).date ():
                    QMessageBox .information (self ,"Activation Status","The software is activated for lifetime.")
                else :
                    QMessageBox .information (self ,"Activation Status",f"The software is activated. Days remaining: {days_remaining}")
                return 

        QMessageBox .information (self ,"Activation Status","The software is not currently activated.")

def main ():
    app =QApplication (sys .argv )


    app .setStyle ("Fusion")


    palette =app .palette ()
    palette .setColor (QPalette .Window ,Qt .white )
    palette .setColor (QPalette .WindowText ,Qt .black )
    palette .setColor (QPalette .Base ,Qt .white )
    palette .setColor (QPalette .AlternateBase ,Qt .lightGray )
    palette .setColor (QPalette .ToolTipBase ,Qt .white )
    palette .setColor (QPalette .ToolTipText ,Qt .black )
    palette .setColor (QPalette .Text ,Qt .black )
    palette .setColor (QPalette .Button ,Qt .lightGray )
    palette .setColor (QPalette .ButtonText ,Qt .black )
    palette .setColor (QPalette .BrightText ,Qt .red )
    palette .setColor (QPalette .Link ,Qt .blue )
    palette .setColor (QPalette .Highlight ,Qt .darkBlue )
    palette .setColor (QPalette .HighlightedText ,Qt .white )
    app .setPalette (palette )

    window =NanoSenseApp ()
    window .show ()
    sys .exit (app .exec ())

if __name__ =="__main__":
    main ()


import sys 
import os 
from PySide6 .QtWidgets import QApplication ,QWidget ,QLabel ,QPushButton ,QCheckBox ,QComboBox ,QListWidget ,QVBoxLayout ,QHBoxLayout ,QFileDialog ,QMessageBox ,QStyleFactory ,QInputDialog ,QDialog 
from PySide6 .QtCore import Qt 
from PySide6 .QtGui import QPalette ,QColor ,QFont 
import numpy as np 
import h5py 
from neo .rawio import AxonRawIO 
import pyabf 

class SDCombineDatasets (QWidget ):
    def __init__ (self ):
        super ().__init__ ()
        self .setWindowTitle ("SD Combine Datasets/Files")
        self .resize (500 ,700 )


        self .app_name_label =QLabel ("SD Combine Datasets/Files")
        self .app_name_label .setAlignment (Qt .AlignCenter )
        self .app_name_label .setFont (QFont ("Arial",16 ,QFont .Bold ))
        self .email_label =QLabel ("shankar.dutt@anu.edu.au")
        self .email_label .setAlignment (Qt .AlignCenter )
        self .select_folder_button =QPushButton ("Select Folder")
        self .include_subfolders_checkbox =QCheckBox ("Include Subfolders")
        self .extension_dropdown =QComboBox ()
        self .extension_dropdown .addItems ([".dataset.npz",".MLdataset.npz",".abf",".h5"])
        self .file_list =QListWidget ()
        self .file_list .setSelectionMode (QListWidget .SelectionMode .MultiSelection )
        self .select_all_checkbox =QCheckBox ("Select All")
        self .same_duration_checkbox =QCheckBox ("Files have the same duration")
        self .same_duration_checkbox .setChecked (True )
        self .folder_path_label =QLabel ()
        self .folder_path_label .setWordWrap (True )
        self .combine_datasets_button =QPushButton ("Combine Datasets/Files")


        main_layout =QVBoxLayout ()
        main_layout .addWidget (self .app_name_label )
        main_layout .addWidget (self .email_label )
        main_layout .addWidget (self .select_folder_button )
        main_layout .addWidget (self .include_subfolders_checkbox )
        main_layout .addWidget (self .extension_dropdown )
        main_layout .addWidget (self .file_list )
        main_layout .addWidget (self .select_all_checkbox )
        main_layout .addWidget (self .same_duration_checkbox )
        main_layout .addWidget (self .folder_path_label )
        main_layout .addWidget (self .combine_datasets_button )

        self .setLayout (main_layout )


        self .select_folder_button .clicked .connect (self .select_folder )
        self .select_all_checkbox .stateChanged .connect (self .select_all )
        self .combine_datasets_button .clicked .connect (self .combine_datasets )

    def select_folder (self ):
        folder_path =QFileDialog .getExistingDirectory (self ,"Select Folder")
        if folder_path :
            self .file_list .clear ()
            self .folder_path_label .setText (f"Selected Folder: {folder_path}")
            extension =self .extension_dropdown .currentText ()
            if self .include_subfolders_checkbox .isChecked ():
                for root ,dirs ,files in os .walk (folder_path ):
                    for file in files :
                        if file .endswith (extension ):
                            file_path =os .path .relpath (os .path .join (root ,file ),folder_path )
                            self .file_list .addItem (file_path )
            else :
                for file in os .listdir (folder_path ):
                    if file .endswith (extension ):
                        self .file_list .addItem (file )

    def select_all (self ,state ):
        is_checked =self .select_all_checkbox .isChecked ()
        for index in range (self .file_list .count ()):
            item =self .file_list .item (index )
            item .setSelected (is_checked )

    def read_abf_file (self ,file_path ):
        try :

            raw_io =AxonRawIO (filename =file_path )


            raw_io .parse_header ()


            channel_index =0 


            signal_size =raw_io .get_signal_size (block_index =0 ,seg_index =0 )



            data =raw_io .get_analogsignal_chunk (block_index =0 ,seg_index =0 ,i_start =0 ,i_stop =signal_size ,channel_indexes =[channel_index ])


            data =raw_io .rescale_signal_raw_to_float (data ,dtype ='float64',channel_indexes =[channel_index ]).flatten ()


            sampling_rate =raw_io .get_signal_sampling_rate ()
            return data ,sampling_rate 
        except :
            abf =pyabf .ABF (file_path )
            data =abf .sweepY 
            sampling_rate =abf .dataRate 
            return data ,sampling_rate 

    def combine_datasets (self ):
        selected_files =[self .file_list .item (i ).text ()for i in range (self .file_list .count ())if self .file_list .item (i ).isSelected ()]
        if selected_files :
            if self .select_all_checkbox .isChecked ():
                selected_files .sort ()
            else :
                selected_indexes =self .file_list .selectedIndexes ()
                selected_files =[self .file_list .item (index .row ()).text ()for index in selected_indexes ]

            extension =self .extension_dropdown .currentText ()
            folder_path =self .folder_path_label .text ().replace ("Selected Folder: ","")

            if extension in [".dataset.npz",".MLdataset.npz"]:
                self .combine_npz_datasets (selected_files ,folder_path ,extension )
            elif extension in [".abf",".h5"]:
                self .combine_abf_h5_datasets (selected_files ,folder_path ,extension )
        else :
            QMessageBox .warning (self ,"Warning","No files selected.")

    def combine_npz_datasets (self ,selected_files ,folder_path ,extension ):
        file_lengths =[]
        if self .same_duration_checkbox .isChecked ():
            file_length ,ok =QInputDialog .getInt (self ,"File Duration","Enter the duration of each file in seconds:")
            if ok :
                file_lengths =[file_length ]*len (selected_files )
            else :
                QMessageBox .warning (self ,"Warning","File duration not provided. Skipping combination.")
                return 
        else :
            for file_path in selected_files :
                file_length ,ok =QInputDialog .getInt (self ,"File Duration",f"Enter the duration of file '{file_path}' in seconds:")
                if ok :
                    file_lengths .append (file_length )
                else :
                    QMessageBox .warning (self ,"Warning","File duration not provided. Skipping file.")
                    return 

        combined_data =None 
        settings_file =None 
        cumulative_time =0 
        for file_path ,file_length in zip (selected_files ,file_lengths ):
            full_path =os .path .join (folder_path ,file_path )
            with np .load (full_path )as data :
                if combined_data is None :
                    combined_data =data ['X']
                else :
                    try :
                        data_x =data ['X']
                        data_x [:,8 ]+=cumulative_time 
                        combined_data =np .concatenate ((combined_data ,data_x ))
                    except :
                        print (f"Cannot save file: {file_path}")
                if settings_file is None :
                    settings_file =data ['settings']
            cumulative_time +=file_length 

        save_path ,_ =QFileDialog .getSaveFileName (self ,"Save Combined Dataset","",f"NumPy Files (*{extension})")
        if save_path :
            if not save_path .endswith (extension ):
                save_path +=extension 
            np .savez (save_path ,X =combined_data ,settings =settings_file )
            QMessageBox .information (self ,"Success","Datasets/Files combined successfully.")

    def combine_abf_h5_datasets (self ,selected_files ,folder_path ,extension ):
        combined_data =[]
        sampling_rate =None 

        for file_path in selected_files :
            full_path =os .path .join (folder_path ,file_path )
            if extension ==".abf":
                data ,file_sampling_rate =self .read_abf_file (full_path )
            elif extension ==".h5":
                with h5py .File (full_path ,'r')as f :
                    data =f ['selected_data'][:]
                    file_sampling_rate =f .attrs ['sampling_rate']

            combined_data .append (data )

            if sampling_rate is None :
                sampling_rate =file_sampling_rate 
            elif sampling_rate !=file_sampling_rate :
                QMessageBox .warning (self ,"Warning","Files have different sampling rates. Proceeding with combination, but please be cautious with the results.")


        combined_data =np .concatenate (combined_data )


        save_path ,_ =QFileDialog .getSaveFileName (self ,"Save Combined Dataset","","HDF5 Files (*.h5)")
        if save_path :
            if not save_path .endswith ('.h5'):
                save_path +='.h5'
            with h5py .File (save_path ,'w')as f :
                f .create_dataset ('selected_data',data =combined_data )
                f .attrs ['sampling_rate']=sampling_rate 
            QMessageBox .information (self ,"Success","Datasets combined successfully and saved as .h5 file.")

if __name__ =='__main__':
    app =QApplication (sys .argv )
    app .setStyle (QStyleFactory .create ('Fusion'))


    palette =QPalette ()
    palette .setColor (QPalette .ColorRole .Window ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .WindowText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Base ,QColor (25 ,25 ,25 ))
    palette .setColor (QPalette .ColorRole .AlternateBase ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .ToolTipBase ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .ToolTipText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Text ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .Button ,QColor (53 ,53 ,53 ))
    palette .setColor (QPalette .ColorRole .ButtonText ,Qt .GlobalColor .white )
    palette .setColor (QPalette .ColorRole .BrightText ,Qt .GlobalColor .red )
    palette .setColor (QPalette .ColorRole .Link ,QColor (42 ,130 ,218 ))
    palette .setColor (QPalette .ColorRole .Highlight ,QColor (42 ,130 ,218 ))
    palette .setColor (QPalette .ColorRole .HighlightedText ,Qt .GlobalColor .black )
    app .setPalette (palette )

    window =SDCombineDatasets ()
    window .show ()
    sys .exit (app .exec ())